#include <bits/stdc++.h>

using namespace std;

const int kMaxN = 500005;
int V[kMaxN];
int n, lo, hi;

pair<int, int> DP[kMaxN];


pair<int, int> unite(pair<int, int> a, pair<int, int> b) {
	if(a.first == -1) return b;
	if(b.first == -1) return a;
	return { min(a.first, b.first), max(a.second, b.second) };
}

pair<int, int> intersect(pair<int, int> a, pair<int, int> b) {
	if(a.first == -1) return a;
	if(b.first == -1) return b;

	auto ret = make_pair(max(a.first, b.first), min(a.second, b.second));
	if(ret.first > ret.second) return {-1, -1};
	return ret;
}


struct ArbInt {
	vector<pair<int, int>> T;

	ArbInt(int n) : T(4 * n, make_pair(-1, -1)) {}

	void Update(int node, int b, int e, int pos, pair<int, int> val) {
		if(b == e) {
			assert(b == pos);
			T[node] = val;
		} else {
			int m = (b + e) / 2;
			if(pos <= m) {
				Update(node * 2, b, m, pos, val);
			} else {
				Update(node * 2 + 1, m + 1, e, pos, val);
			}

			T[node] = unite(T[node * 2], T[node * 2 + 1]);
		}
	}

	pair<int, int> Query(int node, int b, int e, int l, int r) {
		if(b >= l && e <= r) return T[node];
		if(b > r || e < l) return make_pair(-1, -1);

		int m = (b + e) / 2;
		return unite(Query(node * 2, b, m, l, r),
			Query(node * 2 + 1, m + 1, e, l, r));
	}
};
template<typename cmp>
struct Poset {
	vector<int> Pos;
	set<int, cmp> Set;

	Poset(int n) : Pos(n, -1) {}

	void Update(int pos, int val) {
		if(Pos[val] != -1) {
			Set.erase(Pos[val]);
		}
		Pos[val] = pos;
		Set.insert(pos);
	}

	pair<int, int> GetLastTwo() {
		auto it = Set.begin();
		pair<int, int> ret(-1, -1);

		if(it != Set.end()) {
			ret.first = *it;
			++it;
			if(it != Set.end())
				ret.second = *it;
		}

		return ret;
	}
};

int ForwardProp() {
	DP[0] = {0, 0};

	ArbInt ai(n + 2);
	Poset<greater<int>> pos(n + 2);

	ai.Update(1, 0, n, 0, DP[0]);
	for(int i = 1; i <= n; ++i) {
		DP[i] = {-1, -1};

		if(V[i] != -1) pos.Update(i, V[i]);
		auto lasts = pos.GetLastTwo();

		pair<int, int> inter, range;

		inter = intersect(make_pair(max(0, i - hi), i - lo),
			make_pair(lasts.first == -1 ? 0 : lasts.first, i));
		inter = intersect(inter, make_pair(0, n + 1));
		range = ai.Query(1, 0, n, inter.first, inter.second);
		
		if(range.first != -1) {
			DP[i] = unite(DP[i], make_pair(range.first + 1, range.second + 1));
		}

		if(lasts.first != -1) {
			inter = intersect(make_pair(max(0, i - hi), i - lo),
				make_pair(lasts.second == -1 ? 0 : lasts.second, lasts.first - 1));
			inter = intersect(inter, make_pair(0, n + 1));
			range = ai.Query(1, 0, n, inter.first, inter.second);

			if(range.first <= V[lasts.first] - 1 && V[lasts.first] - 1 <= range.second) {
				DP[i] = unite(DP[i], make_pair(V[lasts.first], V[lasts.first]));
			}
		}

		ai.Update(1, 0, n, i, DP[i]);
	}
	return DP[n].second;
}

void BackProp(int maxx) {
	DP[n + 1] = {maxx + 1, maxx + 1};

	ArbInt ai(n + 2);
	Poset<less<int>> pos(n + 2);

	ai.Update(1, 1, n + 1, n + 1, DP[n + 1]);
	for(int i = n; i >= 1; --i) {
		DP[i] = {-1, -1};

		if(V[i] != -1) pos.Update(i, V[i]);
		auto lasts = pos.GetLastTwo();

		pair<int, int> inter, range;

		inter = intersect(make_pair(i + lo, i + hi),
			make_pair(i, lasts.first == -1 ? n + 1 : lasts.first));
		inter = intersect(inter, make_pair(0, n + 1));
		range = ai.Query(1, 1, n + 1, inter.first, inter.second);

		if(range.first != -1) {
			DP[i] = unite(DP[i], make_pair(max(0, range.first - 1), max(0, range.second - 1)));
		}

		if(lasts.first != -1) {
			inter = intersect(make_pair(i + lo, i + hi),
				make_pair(i, lasts.second == -1 ? n + 1 : lasts.second));
			inter = intersect(inter, make_pair(0, n + 1));
			range = ai.Query(1, 1, n + 1, inter.first, inter.second);

			if(range.first <= V[lasts.first] + 1 && V[lasts.first] + 1 <= range.second) {
				DP[i] = unite(DP[i], make_pair(V[lasts.first], V[lasts.first]));
			}
		}

		ai.Update(1, 1, n + 1, i, DP[i]);
	}
}

int main() {
	freopen("startrek.in", "r", stdin);
	freopen("startrek.out", "w", stdout);

	int m;
	assert(cin >> n >> lo >> hi >> m);
	assert(1 <= n && n <= 100000);
	assert(1 <= lo && lo < hi && hi <= n);
	assert(1 <= m && m <= n);

	memset(V, -1, sizeof(V));

    int old = -1;
	while(m--) {
		int pos, val;
		assert(cin >> pos >> val);
		assert(1 <= pos && pos <= n);
		assert(1 <= val && val <= n);
		assert(pos > old);
		old = pos;
		V[pos] = val;
	}

	int maxx = ForwardProp();
	BackProp(maxx);
	
	cout << maxx << endl;
	int pos = 1;
	for(int val = 1; val <= maxx; ++val) {
		int go = -1;
		for(int nxt = pos + lo; nxt <= pos + hi; ++nxt) {
			if(nxt > n + 1) break;

			if(DP[nxt].first != -1 && DP[nxt].first <= val + 1) go = nxt;

			if(V[nxt] != -1 && V[nxt] != val)
				break;
		}
		assert(go != -1);
		while(pos != go) {
			cout << val << " ";
			++pos;
		}
	}
	assert(pos == n + 1);
	cout << endl;


	return 0;
}
